using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.PlayerLoop;

namespace Lightsaber
{
    class Mod
    {
        public static AudioClip[] HitSFX;

        public static void Main()
        {
            HitSFX = new[]
            {
                ModAPI.LoadSound("sfx/hit_1.mp3"),
                ModAPI.LoadSound("sfx/hit_2.mp3"),
                ModAPI.LoadSound("sfx/hit_3.mp3"),
                ModAPI.LoadSound("sfx/hit_4.mp3"),
            };

            AddLightsaber("anakin", AudioPreset.Jedi, Color.cyan);
            AddLightsaber("ahsoka", AudioPreset.Luke, new Color(0.2f, 1, 0));
            AddLightsaber("obi-wan", AudioPreset.Jedi, Color.cyan);
            AddLightsaber("plo koon", AudioPreset.Jedi, new Color(0, 0.57f, 1));
            AddLightsaber("kit fisto", AudioPreset.Luke, Color.green);
            AddLightsaber("rey", AudioPreset.Rey, Color.yellow);
            AddLightsaber("luke", AudioPreset.Luke, Color.green);
            AddLightsaber("yoda", AudioPreset.Luke, Color.green, 1.3f, 0.02f);
            AddLightsaber("mace", AudioPreset.Jedi, new Color(0.847f, 0.141f, 1));
            AddLightsaber("vader", AudioPreset.Sith, Color.red);
            AddLightsaber("palpatine", AudioPreset.Sith, Color.red * 1.5f);
            AddLightsaber("starkiller", AudioPreset.Sith, Color.red);
            AddLightsaber("maul", AudioPreset.Sith, Color.red, 1.8f, 0.4f, true);

            // AddBlaster("E11", ModAPI.LoadSound("sfx/E11.mp3"), Color.red);
            // AddBlaster("DC15s", ModAPI.LoadSound("sfx/DC15s.mp3"), Color.blue);
        }

        private static void AddLightsaber(string name, AudioPreset audiopreset, Color color, float maxSaberLength = 1.5f, float minSaberLength = 0.2f, bool doubleBladed = false)
        {
            var lightsaber = new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Rod"),
                NameOverride = name + "'s lightsaber [zooi]",
                DescriptionOverride = "an elegant weapon for a more civilised age",
                CategoryOverride = ModAPI.FindCategory("Melee"),
                NameToOrderByOverride = "lightsaber",
                ThumbnailOverride = ModAPI.LoadSprite(name + "_view.png"),
                AfterSpawn = (Instance) =>
                {
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite(name + ".png");
                    Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Weapon");
                    Instance.FixColliders();

                    LightsaberBehaviour lightsaberBehaviour = Instance.GetOrAddComponent<LightsaberBehaviour>();
                    lightsaberBehaviour.AudioPreset = audiopreset;
                    lightsaberBehaviour.MaxSaberLength = maxSaberLength;
                    lightsaberBehaviour.MinSaberLength = minSaberLength;
                    lightsaberBehaviour.DoubleBladed = doubleBladed;
                    lightsaberBehaviour.SetColor(color);
                }
            };

            ModAPI.Register(lightsaber);
        }

        private static void AddBlaster(string name, AudioClip sound, Color color)
        {
            var lightsaber = new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Blaster"),
                NameOverride = name + " [zooi]",
                DescriptionOverride = "clumsy and random",
                CategoryOverride = ModAPI.FindCategory("Firearms"),
                NameToOrderByOverride = "blaster starwars",
                ThumbnailOverride = ModAPI.LoadSprite(name + "_view.png"),
                AfterSpawn = (Instance) =>
                {
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite(name + ".png");
                    foreach (Transform child in Instance.transform)
                        if (child.name.Contains("light"))
                            GameObject.Destroy(child.gameObject);
                    Instance.FixColliders();
                    GameObject.Destroy(Instance.GetComponent<BlasterBehaviour>());
                    var blaster = Instance.GetOrAddComponent<StarWarsBlasterBehaviour>();
                    blaster.Clip = sound;
                    blaster.Color = color;
                    // blaster.blasterSoundSource.clip = ModAPI.LoadSound("sfx/" + name + ".mp3");
                }
            };

            ModAPI.Register(lightsaber);
        }
    }

    public static class LinqReplacement
    {
        // sick ass replacement for LINQ
        public static void RemoveAll<TKey, TValue>(this Dictionary<TKey, TValue> dictionary, Func<TValue, bool> filter)
        {
            List<TKey> toRemove = new List<TKey>();
            foreach (var pair in dictionary)
                if (filter(pair.Value))
                    toRemove.Add(pair.Key);

            foreach (var key in toRemove)
                dictionary.Remove(key);
        }
    }

    [System.Serializable]
    public struct AudioPreset
    {
        public const string PathPrefix = "sfx/";

        public readonly AudioClip On;
        public readonly AudioClip Off;
        public readonly AudioClip Idle;
        public readonly AudioClip Swing;

        public AudioPreset(string on, string off, string idle = "idle.wav", string swing = "swing.wav")
        {
            On = ModAPI.LoadSound(PathPrefix + on);
            Off = ModAPI.LoadSound(PathPrefix + off);
            Idle = ModAPI.LoadSound(PathPrefix + idle);
            Swing = ModAPI.LoadSound(PathPrefix + swing);
        }

        public static AudioPreset Jedi { get { return new AudioPreset("light_on.mp3", "light_off.mp3", "idle_a.wav"); } }
        public static AudioPreset Sith { get { return new AudioPreset("dark_on.mp3", "dark_off.mp3"); } }
        public static AudioPreset Luke { get { return new AudioPreset("luke_ignite.mp3", "light_off.mp3"); } }
        public static AudioPreset Rey { get { return new AudioPreset("rey_on.mp3", "rey_off.mp3"); } }
    }

    public struct Line
    {
        public Vector2 Start;
        public Vector2 End;

        public Line(Vector2 a, Vector2 b)
        {
            Start = a;
            End = b;
        }

        public bool Intersects(Line line)
        {
            return false;
        }
    }
}
